@@ -2,24 +2,28 @@
 This module contains functions for common path operations.
 """
 
+from __future__ import annotations
+
+import errno
+import fcntl
+import itertools
+
 # system imports
 import os
 import os.path as osp
-import errno
-import shutil
-import itertools
-import unicodedata
-import fcntl
 import platform
+import shutil
+import unicodedata
 from stat import S_ISDIR
-from typing import List, Optional, Tuple, Callable, Iterator, Iterable, Union
+from typing import Callable, Iterable, Iterator, List, Optional, Tuple, Union
 
 # third party imports
 import xattr
 
+from ..constants import IS_LINUX
+
 # local imports
 from .hashing import DropboxContentHasher
-from ..constants import IS_LINUX
 
 F_GETPATH = 50
 
@@ -34,7 +38,9 @@ _AnyPath = Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]
 # ==== path relationships ==============================================================
 
 
-def is_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
+def is_child(
+    path: str | bytes, parent: str | bytes, case_sensitive: bool = True
+) -> bool:
     """
     Checks if ``path`` semantically is inside ``parent``. Neither path needs to
     refer to an actual item on the drive. This function is case-sensitive.
@@ -47,6 +53,9 @@ def is_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
     if not case_sensitive:
         path = normalize(path)
         parent = normalize(parent)
+    else:
+        path = os.fsdecode(path)
+        parent = os.fsdecode(parent)
 
     parent = parent.rstrip(osp.sep) + osp.sep
     path = path.rstrip(osp.sep)
@@ -54,7 +63,9 @@ def is_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
     return path.startswith(parent)
 
 
-def is_equal_or_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
+def is_equal_or_child(
+    path: str | bytes, parent: str | bytes, case_sensitive: bool = True
+) -> bool:
     """
     Checks if ``path`` semantically is inside ``parent`` or equals ``parent``. Neither
     path needs to refer to an actual item on the drive. This function is case-sensitive.
@@ -65,11 +76,7 @@ def is_equal_or_child(path: str, parent: str, case_sensitive: bool = True) -> bo
     :returns: ``True`` if ``path`` semantically lies inside ``parent`` or
         ``path == parent``.
     """
-    if not case_sensitive:
-        path = normalize(path)
-        parent = normalize(parent)
-
-    return is_child(path, parent) or path == parent
+    return is_child(path, parent, case_sensitive) or path == parent
 
 
 # ==== case sensitivity and normalization ==============================================
@@ -96,7 +103,7 @@ def normalize_unicode(string: str) -> str:
     return unicodedata.normalize("NFC", string)
 
 
-def normalize(string: str) -> str:
+def normalize(path: str | bytes) -> str:
     """
     Replicates the path normalization performed by Dropbox servers. This typically only
     involves converting the path to lower case, with a few (undocumented) exceptions:
@@ -116,10 +123,10 @@ def normalize(string: str) -> str:
 
     Todo: Follow Python 2.5 / Dropbox conventions instead of Python 3 conventions.
 
-    :param string: Original path.
+    :param path: Original path.
     :returns: Normalized path.
     """
-    return normalize_case(normalize_unicode(string))
+    return normalize_case(normalize_unicode(os.fsdecode(path)))
 
 
 def is_fs_case_sensitive(path: str) -> bool:
@@ -415,7 +422,7 @@ def move(
 
 
 def walk(
-    root: str,
+    root: str | bytes,
     listdir: Callable[[str], Iterable["os.DirEntry[str]"]] = os.scandir,
 ) -> Iterator[Tuple[str, os.stat_result]]:
     """
@@ -425,7 +432,7 @@ def walk(
     :param listdir: Function to call to get the folder content.
     :returns: Iterator over (path, stat) results.
     """
-    for entry in listdir(root):
+    for entry in listdir(os.fsdecode(root)):
         try:
             path = entry.path
             stat = entry.stat(follow_symlinks=False)
